import type { TypedFormData } from "@k1eu/typed-formdata";
import type { SuperHeadersInit } from "@mjackson/headers";
type IntegrityAlgo = "sha256" | "sha384" | "sha512";
type RequestMethod = "GET" | "HEAD" | "POST" | "PUT" | "DELETE" | "OPTIONS" | "PATCH";
type FormDataInReqeuest<T> = T extends TypedFormData<infer U> ? Promise<TypedFormData<U>> : T extends FormData ? Promise<FormData> : never;
export type TypedRequestInit = Omit<RequestInit, "integrity" | "method"> & {
    headers?: SuperHeadersInit;
    integrity?: `${IntegrityAlgo}-${string}`;
    method?: RequestMethod;
};
type TypedResponseInit = ResponseInit & {
    headers?: SuperHeadersInit;
};
type JSONInResponse<T> = T extends FormData | string ? never : Promise<T>;
type TextInResponse<T> = T extends string ? Promise<T> : never;
export declare class TypedResponse<T extends Record<string, unknown> | FormData | TypedFormData<any> | string = {}> extends Response {
    response: Response;
    constructor(body?: BodyInit | null, init?: TypedResponseInit);
    json(): JSONInResponse<T>;
    text(): TextInResponse<T>;
    formData(): FormDataInReqeuest<T>;
}
export declare class TypedRequest<T extends Record<string, unknown> | FormData | TypedFormData<any> | string = {}> extends Request {
    request: Request;
    constructor(input: RequestInfo | URL, init?: TypedRequestInit);
    json(): JSONInResponse<T>;
    text(): TextInResponse<T>;
    formData(): FormDataInReqeuest<T>;
}
export {};
